{% extends "sveedocuments/page_details/subbase.html" %}
{% load i18n documents_markup documents_utils %}

{% block sveedocuments_content %}<div{% if page_instance.level == 0 %} class="doc_root_page"{% endif %}>
    {% include "sveedocuments/page_details/_action_menu.html" %}
    {% document_context page_instance %}

    <div class="restructuredtext_container">
        {{ page_instance|document_render }}
    </div>
    
    {% if attachments %}
    <h3 id="attachments">{% trans "Attachments" %}</h3>
    <ul>
        {% for attach in attachments %}
        <li><a href="{{ item.file.url }}">{{ attach.title }}</a> {{ attach.size|filesizeformat }}</li>
        {% endfor %}
    </ul>
    {% endif %}
</div>{% endblock %}
